/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*								*/
/*	ACCES AUX IMAGES .lena					*/
/*								*/
/*	FICHIER	re.mj/bib/acces_lena.c				*/
/*	DATE	13 decembre 1991				*/
/*								*/
/*	Ce fichier contient les procedures :			*/
/*	ouvre_lena	()					*/
/*	ecrit_lena	()					*/
/*	lit_lena	()					*/
/*	ferme_lena	()					*/
/*								*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

# include	<stdio.h>
# include	"structlena.h"

/****************************************************************/
/*								*/
/*	Ouverture d'un fichier .lena				*/
/*								*/
/****************************************************************/

image	ouvre_lena	(mode, dsc, nomfich, status)

/* La fonction renvoie une structure 'image' associee au 
   fichier ouvert	*/

/* Arguments */

char	mode;
	/* mode ='e' : 	ouverture en ecriture sans verification 
			de l'existence du fichier
		='w' :	ouverture en ecriture avec verification
			de l'existence du fichier
		='l' :	ouverture en lecture		*/

descripteur *dsc;
	/* pointeur de structure logique en tete fichier .lena	*/
	
char	*nomfich;	/* nom du fichier */

int	*status;
	/*	status = 1  :	ouverture sans erreur
		status = 0  :	mode d'ouverture inconnu
		status = -1 :	ouverture en lecture : image inexistante
		status = -2 :	image deja existante	*/


{
/* Variables locales */

	FILE	*fp, *fopen ();
	image	nouvimag;
	int	fd;


/* Ouverture du fichier 'image' */

	fp = NULL;
	*status = 1;

	if	((mode != 'l') && (mode != 'e') && (mode != 'w'))
	{	*status = 0;
		return	(nouvimag);
	}

	if	(mode == 'l')

	{	if	((fd=open(nomfich,0)) == -1)

		{	*status = -1;
			return	(nouvimag);
		}
	}

	if	(mode =='w')

	{	if	((fd=open(nomfich,0))!= -1)

		{	*status = -2;
			return	(nouvimag);
		}
	}

/* Lecture ou ecriture des caracteristiques de l'image */

	switch	(mode)

	{

	case 'l' :

	   {	fp = fopen (nomfich,"r");
		fscanf	(fp,"%d",&(dsc->nx));
		fseek	(fp,6,0); 
		fscanf	(fp,"%d",&(dsc->ny));
		fseek	(fp,12,0); 
		fscanf	(fp,"%d",&(dsc->nz));
		fseek	(fp,16,0); 
		fscanf	(fp,"%d",&(dsc->typ));
		fseek	(fp,18,0);
		fscanf	(fp,"%14s",dsc->nature);
		fseek	(fp,32,0);
		fscanf	(fp,"%64s",dsc->chemin);
		fseek	(fp,96,0);
		fscanf	(fp,"%32s",dsc->libre);
		fseek	(fp,128,0);
		fscanf	(fp,"%128s",dsc->comment);
		fseek	(fp,256,0);
		nouvimag.p_dec = dsc;
		nouvimag.p_octet = (FILE *)fp;
		return	(nouvimag);
	   }

	case 'e' : case 'w' :

	   {	fp = fopen (nomfich,"w");
		fprintf	(fp,"%5d ",dsc->nx);
		fprintf	(fp,"%5d ",dsc->ny);
		fprintf	(fp,"%3d ",dsc->nz);
		fprintf	(fp,"%1d ",dsc->typ);
		fseek	(fp,18,0);
		fprintf	(fp,"%-13s ",dsc->nature);
		fseek	(fp,32,0);
		fprintf	(fp,"%-63s ",dsc->chemin);
		fseek	(fp,96,0);
		fprintf	(fp,"%-31s ",dsc->libre);
		fseek	(fp,128,0);
		fprintf	(fp,"%-127s ",dsc->comment);
		fseek	(fp,256,0);
		nouvimag.p_dec = dsc;
		nouvimag.p_octet = (FILE *)fp;
		return	(nouvimag);
	   }
	}
}


/****************************************************************/
/*								*/
/*	Ecriture d'un fichier .lena				*/
/*								*/
/****************************************************************/


int	ecrit_lena	(p_im, im, nblig)

/* Arguments */

image	*p_im;
	/* structure logique associee au fichier .lena, initialisee
	   lors de l'ouverture */

unsigned char	*im;	/* tableau image */

int	nblig;		/* nombre de lignes a ecrire */

/* La fonction renvoie :
		status = 1   :	ecriture sans erreur
		status = EOF :	erreur rencontree	*/

{
/* Variables locales */

	int	k=0, n_octet;
	int	status=1;

/* Ecriture du fichier */

	n_octet = nblig*(p_im->p_dec->nx)*(p_im->p_dec->nz);

	do
	{	status = putc	(im[k],(FILE *)p_im->p_octet);
		++k;
	}	while	((k < n_octet) && (status != EOF));

	return	(status);
}


/****************************************************************/
/*								*/
/*	Lecture d'un fichier .lena				*/
/*								*/
/****************************************************************/


int	lit_lena	(p_im, im, nblig)

/* Arguments */

image	*p_im;
	/* structure logique associee au fichier .lena, initialisee
	   lors de l'ouverture */

unsigned char	*im;	/* tableau accueillant l'image */

int	nblig;		/* nombre de lignes a lire */

/* La fonction renvoie :
		status = 1  :	lecture sans erreur
		status = -1 :	erreur rencontree	*/

{
/* Variables locales */

	int	k=0, n_octet;
	int	status = 1;

/* Lecture du fichier */

	n_octet = nblig*(p_im->p_dec->nx)*(p_im->p_dec->nz);

	do
	{	im[k] = (unsigned char) getc ((FILE *)p_im->p_octet);
		++k;
	}	while	((k < n_octet) && (im[k-1] != EOF));

	if	(k < n_octet)
		status = -1;

	return	(status);
}


/****************************************************************/
/*								*/
/*	Fermeture d'un fichier .lena				*/
/*								*/
/****************************************************************/


int	ferme_lena	(p_im)

image	*p_im;
	/* pointeur de la structure logique associee au fichier .lena */

/* La fonction renvoie :
		status = 1  :	fermeture sans erreur
		status = -1 :	erreur rencontree	*/

{
	if	(fclose	((FILE *)p_im->p_octet) == EOF)
		return	(-1);
	else
		return	(1);
}
