#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "structlena.h"

#define tx 35 // 1/2 de la largeur du filtre
#define ty 35 // 1/2 de la hauteur du filtre

/*
Didier STRAUS & Julien GONY

Compilation : make

Execution : ./progabor

Gabor v1.0  29/04/04 - 14/05/04
*/

typedef struct complex {double re;double im;} complex;

/* *** prototypes externes *** */
extern image ouvre_lena();
extern lit_lena();
extern ecrit_lena();
extern ferme_lena();

/* *** prototypes internes *** */
complex gabor(int x,int y,double fx0,double fy0);
void calculMat(int sigma,double fx0,double fy0);
void appliquerFiltre();

/* *** variables globales *** */
int SIGMA;
double tTFD[tx][ty];
double tTFDg[tx][ty];
complex matFiltre[tx*2][ty*2];
descripteur desc,desc2;
image img,img2;
int etat,etat2;
char mode='l';
unsigned char *tab;
unsigned char *tab2;

/* ************************************************* */

int main(void){
int i,j,k,p;
char lena[6]=".lena\0";
char nameLena[35];
char nom[30];
float freq[8][2]={ /* les frequences fx0 et fy0 */
 {0.2  ,0.0},
 {0.4  ,0.0},
 {0.15 ,0.15},
 {0.3  ,0.3},
 {0.0  ,0.2},
 {0.0  ,0.4},
 {-0.15,0.15},
 {-0.3 ,0.15}
};

printf("	Gabor v1.0 par Didier STRAUS & Julien GONY\n");

do {
 printf("\nSaisissez le nom de votre image (sans l'extension .lena) :\n");
 scanf("%30s",&nom); // le nom doit faire moins de 30 caracteres
} while(nom==""); // le nom ne doit pas etre vide

strcpy(nameLena,nom);      // on fait une copie du nom
strncat(nameLena,lena,35); // on ajoute l'extension .lena a la copie du nom

printf("\nOuverture de l'image %s\n",nameLena);
img=ouvre_lena(mode,&desc,nameLena,&etat);

if (etat==0) {
 printf("Erreur : mode d'ouverture inconnu\n");
 exit(0);
}
else if (etat==-1) {
 printf("Erreur : ouverture en lecture : image inexistante\n");
 exit(0);
}


do { // choix de Sigma
 printf("\nSaisissez la valeur de Sigma (2-11) (pref:2) :\n");
 scanf("%d",&SIGMA);
} while((SIGMA<2) || (SIGMA>11));


do { // choix du coefficient des frequences fx0 et fy0
 printf("\nSaisissez le coefficient des frequences (1-10) (pref:1) :\n");
 scanf("%d",&k);
} while((k<1) || (k>10));

tab=malloc(desc.nx*desc.ny*desc.nz); // on alloue la memoire necessaire pour stocker l'image source
tab2=malloc(desc.nx*desc.ny*desc.nz);// on alloue la memoire necessaire pour stocker l'image filtre


etat=1;
etat=lit_lena(&img,tab,desc.ny);

if (etat==1)
 printf("\nDebut du traitement...\n");
else
 printf("\nUn probleme est survenu.\n"); // en toute logique, ca ne doit jamais s'afficher

/* calcul intermediare */

printf("Taille image : %d x %d\n",desc.nx,desc.ny);


for(p=1;p<9;p++) { // on va appliquer 8 filtres sur l'image
 char nameSave[36];
 int max=0;

 strcpy(nameSave,nom); // on fait une copie du nom

 switch(p) { // on ajoute le chiffre
  case 1: strncat(nameSave,"1",36);
			break;

  case 2: strncat(nameSave,"2",36);
			break;

  case 3: strncat(nameSave,"3",36);
			break;

  case 4: strncat(nameSave,"4",36);
			break;

  case 5: strncat(nameSave,"5",36);
			break;

  case 6: strncat(nameSave,"6",36);
			break;

  case 7: strncat(nameSave,"7",36);
			break;

  case 8: strncat(nameSave,"8",36);
			break;

  case 9: strncat(nameSave,"9",36);
			break;
 }

 strncat(nameSave,lena,36); // on ajoute l'extension .lena

 for(i=0;i<desc.nx*desc.ny;i++) // initialistaion de l'image resultat en noire
  tab2[i]=0;

 /* Calcul du filtre */
 calculMat(SIGMA, freq[p-1][0]*k, freq[p-1][1]*k);

 /* Application du filtre sur l'image */
 appliquerFiltre();

 /* recadrage dynamique */
 for(i=0;i<desc.nx*desc.ny;i++) { /* recherche de la valeur maximum d'un pixel */
  if (tab2[i]>max)
   max=tab2[i];
 }

 /* modification de l'image (recadrage) */
 for(i=0;i<desc.nx*desc.ny;i++) {
  tab2[i]=tab2[i]*(255/max);
 }

 /* sauvegarde du fichier */
 printf("%s\n",nameSave);
 img2=ouvre_lena('e',&desc,nameSave,&etat2);

 etat= ecrit_lena(&img2,tab2,desc.ny);
 ferme_lena(&img2); /* on ferme l'image generee */

}


ferme_lena(&img); /* on ferme l'image source */

printf("Fin du traitement.\n");
return(0);
} /* main */


/* ***************************************************** */

complex gabor(int x,int y,double fx0,double fy0) {
 double a,res,cc,ss;
 complex c;

 cc=cos(2.*M_PI*(fx0*(double)x+fy0*(double)y));
 ss=sin(2.*M_PI*(fx0*(double)x+fy0*(double)y));

 c.re = (1/(2*M_PI*SIGMA*SIGMA))*exp((-(x*x)-(y*y))/(2*SIGMA*SIGMA))*cc;
 c.im = (1/(2*M_PI*SIGMA*SIGMA))*exp((-(x*x)-(y*y))/(2*SIGMA*SIGMA))*ss;

 return c;
} /* gabor */


/* ***************************************************** */

void calculMat(int sigma,double fx0,double fy0) {
 int i,j;
 complex c;

 for(i=0;i<3*sigma;i++) {
  for(j=0;j<3*sigma;j++) {
   c=gabor(i,j,fx0,fy0);

   matFiltre[((3*sigma)-1)+i][((3*sigma)-1)+j].re = c.re; // on cree la partie a droite en bas
   matFiltre[((3*sigma)-1)+i][((3*sigma)-1)+j].im = c.im; // on cree la partie a droite en bas
  }
 }

 for(i=0;i<((3*sigma));i++) { // on cree la partie en bas a gauche (symetrique par rapprot a la partie droite)
  for(j=0;j<((3*sigma));j++) {
   matFiltre[((3*sigma)-1)+i][((3*sigma)-1)-j].re=matFiltre[((3*sigma)-1)+i][((3*sigma)-1)+j].re;
   matFiltre[((3*sigma)-1)+i][((3*sigma)-1)-j].im=matFiltre[((3*sigma)-1)+i][((3*sigma)-1)+j].im;
  }
 }

 for(i=0;i<(3*sigma);i++) { // on cree la partie en haut (droite et gauche) (symetrique par rapport au bas)
  for(j=0;j<(2*3*sigma);j++) {
   matFiltre[((3*sigma)-1)-i][j].re=matFiltre[((3*sigma)-1)+i][j].re;
   matFiltre[((3*sigma)-1)-i][j].im=matFiltre[((3*sigma)-1)+i][j].im;
  }
 }

} /* calculMat */


/* ***************************************************** */

void appliquerFiltre() {
 int i,j,k,p,q;

 // pour chaque pixel de l'image
 for(i=(3*SIGMA)-1;i<desc.ny-((3*SIGMA)-1);i++) {  // colonne
  for(j=(3*SIGMA)-1;j<desc.nx-((3*SIGMA)-1);j++) { // ligne
   complex som;

   som.re=som.im=0; // initialisation
   // on applique le filtre sur 1 pixel
   for(p=-((3*SIGMA)-1);p<=((3*SIGMA)-1);p++) { // calcul l'effet du filtre sur la zone voisine du pixel
    for(q=-((3*SIGMA)-1);q<=((3*SIGMA)-1);q++) {
     som.re+=tab[(i+p)*desc.nx+(j+q)] * matFiltre[p+(3*SIGMA)-1][q+(3*SIGMA)-1].re;
     som.im+=tab[(i+p)*desc.nx+(j+q)] * matFiltre[p+(3*SIGMA)-1][q+(3*SIGMA)-1].im;
    }
   }

   tab2[i*desc.nx+j]=(int)sqrt(som.re*som.re+som.im*som.im); // on memorise le pixel filtre
  }
 }

} /* appliquerFiltre */
